program compute_G
  !==================================================================================================
  ! this program computes VanRaden's 2008 or Yang et al. (Nature genetics 2010) G-matrix 
  ! coefficients for genomic selection data in UGA format
  !==================================================================================================
  ! Modification alegarra+guillaume salle  23/01/2011 : integration inversion G
  !==================================================================================================
  ! considers missing genotypes coded as 5
  use ieee_arithmetic
  implicit none
  integer,parameter:: maxsnp=500000, dp=kind(1d0)
  integer:: i,nsnp,io,nanim,k,new_nsnp,pos,j,coun,ip_snp,rank,method
  integer,allocatable:: geno(:,:),new_geno(:,:),id(:)
  character(len=100):: genofile,outfile1,outfile2,informat
  character(len=2)::dum
  real(dp):: t,F,xij
  real(dp),allocatable:: freq(:),MAF(:)
  real(dp),allocatable:: X(:,:),G(:,:),XD(:,:)
  character(len=maxsnp):: line
  real:: t1,t2
  logical:: writeOut,writeOutGi


  ! first read to find out number of SNPs
  ! and number of individuals
  print *,'genofile?'
  read *,genofile
  print *,'out file for G :'
  outfile1=adjustl(trim(genofile))//'.G'
  print *,outfile1
  !read *,outfile1
  open (2,file=outfile1,status='replace')
  print *,'out file for G-1 :'
  outfile2=adjustl(trim(genofile))//'.Gi'
  print *,outfile2
  !read *,outfile2
  open (3,file=outfile2,status='replace')
  print *,'which G?'
  do
	  print *,' 1 - VanRaden firstG = ZZ''/sum(2pq)'
	  print *,' 2 - VanRaden secondG = Yang et al. = mean (Z_i Z_i'' /(2p_i q_i))'
	  read  *,method
	  if(method==1 .or. method==2) exit
  enddo
  print *,'write out G ? (T:F)'
  read *,writeOut
  print *,'write out G inverse? (T:F)'
  read *,writeOutGi


  !==============================================================================
  ! Reading geno file
  ! this whole section is from I Aguilar
  ! determine Number of snp and postion of first marker in SNP_file
  !
  ! it is further assumed that the id is in the middle (AL)
  !==============================================================================
  i=0
  call cpu_time(t1)
  open(1,file=genofile,status='old')

  rewind(1)
  read(1,'(a)') line
  ip_snp=index(line(1:50)," ",back=.true.)+1
  nSnp=index(line(ip_snp:)," ")-1
  write(informat,'(a,i0,a,i0,a)') '(i',ip_snp-2,',1x,',nsnp,'i1)'
  if (nSNP==-1) then
     print '(a,i0,a)', 'Number of SNPs greater than ', maxsnp,&
          ', or error in reading SNP file'
     stop
  endif
  print*,'Column position in file for the first marker: ',ip_snp
  print*,'Format to read SNP file: ',informat
  print*,'Number of SNPs :', nSNP
  rewind(1)
  i=0
  do 
     read(1,*,iostat=io) k
     if(io/=0) exit
     i=i+1
  enddo
  nanim=i
  print *,'nanim= ',nanim,'nsnp= ',nsnp
  allocate(geno(nanim,nsnp),G(nanim,nanim),X(nanim,nsnp),&
       freq(nsnp),MAF(nsnp),id(nanim),XD(nanim,nsnp))
  rewind(1)
  !write(informat,'(i10)')nsnp
  !print *,informat
  !informat='(2i10,1x,'//trim(adjustl(informat))//'i1)'
  !print *,informat

  do i=1,nanim
     !read(1,informat) id(i),nsnp,geno(i,:)
     read(1,informat) id(i),geno(i,:)
     if(mod(i,100)==0) print *,i
  enddo
  call cpu_time(t2)
  print *,'data read, time',t2-t1
  call cpu_time(t1)
  X=0d0
  do i=1,nsnp
     if(mod(i,10000)==0) print *,i,'ith SNP'
     pos=i ! position of the first allele
     where(geno(:,i)==0)
        X(:,pos)=0d0
     elsewhere(geno(:,i)==1)
        X(:,pos)=1d0
     elsewhere(geno(:,i)==2)
        X(:,pos)=2d0
     end where
  enddo
  print *,'X setup'
  ! deviate X from average within-individual value
  do i =1,nsnp
     ! how many observed genotypes?
     if(mod(i,10000)==0d0) print *,i,'ith SNP'
     freq(i)=sum(X(:,i))/(2d0*count(geno(:,i)/=5))
     where(geno(:,i)/=5) ! if observed; if missing, set to zero
       X(:,i) = X(:,i)-2d0*freq(i)
     endwhere
  enddo
  ! polymorphic SNPs
  coun=count( (freq*(1d0-freq)) > 0.05 )
  print *,'coun',coun

  print *,'average freq',mean(freq),'var(freq)',var(freq)
  print *,'X re-setup'

  !=================================================
  ! compute X D X' where D is diag(freq*(1-freq))
  ! Ignacio Aguilar - like style
  ! This is PVR "second G"
  ! too slow!!
  !=================================================
  !do i=1,nanim
  !  do j=1,nanim
  !    do k=1,nsnp
  !      if( (freq(k)*(1d0-freq(k))) >0) then
  !	      G(i,j) = G(i,j) + X(i,k)*X(j,k)/(2d0*freq(k)*(1d0-freq(k)))
  !      endif
  !    enddo
  !  enddo
  !enddo

  if(method==2) then

	  !------ Compute  VanRaden's second G = Yang
	  XD=0d0
	  do i=1,nsnp
	     if( (freq(i)*(1d0-freq(i))) > 0.05) then
		XD(:,i)=X(:,i)/sqrt(2d0*freq(i)*(1d0-freq(i)))
	     endif
	  enddo
	  G=matmul(XD,transpose(XD))
	  G=G/coun
  elseif(method==1) then
  
	  !------ Compute Van Raden first G
	  G=matmul(X,transpose(X))/ (2.d0*sum(freq*(1d0-freq)))
  endif

  ! modify G=0.95G+0.05I to make G invertible (it usually is not)
  
  G=G*0.95
  do i=1,nanim
     G(i,i)=G(i,i) + 0.05
  enddo
  
  call cpu_time(t2)
  print *,'G computed, time',t2-t1

  if(writeOut) then
	  do i=1,nanim
	     do j=1,nanim
		   write(2,*) i,j,id(i),id(j),G(i,j)
	     enddo
	  enddo
  endif
  !=================================
  ! G inversion
  !=================================
  call cpu_time(t1)
  ! Subroutine ginv: (matrice,nanim,tolerance,rang)
  call ginv(G,nanim,1d-50,rank)
  call cpu_time(t2)
  print *,'G-1 computed, time',t2-t1
  print *,'Rang G: ', rank
  if(writeOutGi) then
	  do i=1,nanim
	     do j=1,nanim
		write(3,*) i,j,id(i),id(j),G(i,j)
	     enddo
	  enddo
  endif
  !--------------------------------
contains

  function var(x)
    !variance of x, brutal force
    implicit none
    real(dp)::x(:),mean,var

    mean=sum(x)/size(x)
    var=dot_product(x,x)/size(x) -mean**2
  end function var


    function mean(x)
    !mean of x, brutal force
    implicit none
    real(dp)::x(:),mean
  
   mean=sum(x)/size(x)
    end function

	subroutine ginv(a,n,tol,rank)
	! returns generalized inverse of x(n,n). tol is working zero 
	! and irank returns the rank of the matrix. rework of rohan fernando's 
	! f77 subroutine by i. misztal 05/05/87-05/23/00

	 implicit none
	 integer n
	 real(dp) a(n,n),w(n),tol
	 integer i,ii,j,rank
	 
	 rank=n
	 do i=1,n    
	    do  j=1,i-1
		 a(i:n,i)=a(i:n,i)-a(i,j)*a(i:n,j) 
	    enddo
	    if (a(i,i).lt.tol) then
		 a(i:n,i)=0.0	      
	      rank=rank-1
	      else
		a(i,i)=sqrt(a(i,i))
		a(i+1:n,i)=a(i+1:n,i)/a(i,i)
	    endif
	 enddo	 

	 do i=1,n
	    if (a(i,i).eq.0.) then
		 a(i+1:n,i)=0
	      else
		 a(i,i)=1.0/ a(i,i)
		 w(i+1:n)=0
		 do  ii=i+1,n                  
		     w(ii:n)=w(ii:n)-a(ii:n,ii-1)*a(ii-1,i)
		     if (a(ii,ii).eq.0.) then
			 a(ii,i)=0.
		       else
			 a(ii,i)=w(ii)/a(ii,ii)
		     endif
		 enddo
	     endif
	 enddo

	 do j=1,n
	    do i=j,n
	  a(i,j)=dot_product(a(i:n,j),a(i:n,i))
	    enddo
	 enddo

	 do i=1,n
	    a(i,i+1:n)=a(i+1:n,i)
	 enddo
	 
	 end subroutine

end program compute_G





