program filter_maf
! this program takes a genotype file in USDA format and filters alleles for a minimum
! MAF threshold x producing another genotype file called genotype_file_MAFx
! if x<0, it does not produce any file but outputs statistics
!
! it also computes allelic frequencies in a form useful for BayLa
! corrected for lack of consideration of missing genotypes
implicit none
integer,parameter:: dp=kind(1d0),maxsnp=500000
integer:: i,nsnp,io,nanim,k,new_nsnp,pos,ngeno,ip_snp,aux
integer,allocatable:: geno(:,:),new_geno(:,:),id(:)
character(len=100):: genofile,outfile,informat
character(len=2)::dum
real(dp):: t
real(dp),allocatable:: freq(:),MAF(:),mis(:)
character(len=maxsnp):: line


! first read to find out number of SNPs
! and number of individuals
print *,'genofile?'
read *,genofile
print *,'minimum MAF?'
read *,t

  i=0
  open(1,file=genofile,status='old')

  rewind(1)
  read(1,'(a)') line
  ip_snp=index(line(1:50)," ",back=.true.)+1
  nSnp=index(line(ip_snp:)," ")-1
  write(informat,'(a,i0,a,i0,a)') '(i',ip_snp-2,',1x,',nsnp,'i1)'
  if (nSNP==-1) then
     print '(a,i0,a)', 'Number of SNPs greater than ', maxsnp,&
          ', or error in reading SNP file'
     stop
  endif
  print*,'Column position in file for the first marker: ',ip_snp
  print*,'Format to read SNP file: ',informat
  print*,'Number of SNPs :', nSNP
  rewind(1)
  i=0
  do 
     read(1,*,iostat=io) k
     if(io/=0) exit
     i=i+1
  enddo
  nanim=i
  print *,'nanim= ',nanim,'nsnp= ',nsnp
  allocate(geno(nanim,nsnp),mis(nsnp),&
       freq(nsnp),MAF(nsnp),id(nanim))
  rewind(1)
  !write(informat,'(i10)')nsnp
  !print *,informat
  !informat='(2i10,1x,'//trim(adjustl(informat))//'i1)'
  !print *,informat

  do i=1,nanim
     !read(1,informat) id(i),nsnp,geno(i,:)
     read(1,informat) id(i),geno(i,:)
     if(mod(i,1000)==0) print *,i
  enddo
  print *,'data read'

! compute MAF's
do i =1,nsnp
  if(mod(i,10000)==0) print *,i
  ngeno=count(geno(:,i)/=5) ! not missing
  freq(i)=sum(real(geno(:,i),dp),mask=(geno(:,i)/=5))/(2d0*ngeno)
  maf(i)=min(freq(i),1d0-freq(i))
  mis(i)=count(geno(:,i)==5)
enddo
print *,'missing genotypes',sum(mis)/nanim
print *,'average freq= ',sum(freq)/nsnp
print *,'sdfreq= ',sqrt( sum(freq**2)/nsnp - (sum(freq)/nsnp)**2 )
print *,'maxfreq= ',maxval(freq)
print *,'minfreq= ',minval(freq)
print *,'average MAF= ',sum(MAF)/nsnp
print *,'sdMAF= ',sqrt( sum(MAF**2)/nsnp - (sum(MAF)/nsnp)**2 )
print *,'maxMAF= ',maxval(MAF)
print *,'minMAF= ',minval(MAF)
new_nsnp=count(maf>t)
print *,'number of SNP over MAF>',t,' is ',new_nsnp 
allocate(new_geno(nanim,new_nsnp))
new_geno=5 ! missing value in USDA format
pos=0
open(4,file=trim(adjustl(genofile))//'_freq',status='replace')
write(4,*)'freq oldpos newpos'
do i=1,nsnp
  aux=0
  if(maf(i)>t) then
    pos=pos+1
    new_geno(:,pos)=geno(:,i)
  endif
  aux=pos
  write(4,*) freq(i),i,aux
enddo
close(4)

! provide some information
deallocate(freq); deallocate(maf)
allocate(freq(new_nsnp),maf(new_nsnp))
freq=0d0
maf=0d0
do i =1,new_nsnp
  if(mod(i,10000)==0) print *,i
  ngeno=count(new_geno(:,i)/=5) ! not missing
  freq(i)=sum(real(new_geno(:,i),dp),mask=(new_geno(:,i)/=5))/(2d0*ngeno)
  !freq(i)=sum(real(new_geno(:,i),dp))/(2*nanim)
  maf(i)=min(freq(i),1d0-freq(i))
enddo

print *, ' -- After cleaning out by MAF -- '
print *,'average freq= ',sum(freq)/nsnp
print *,'sdfreq= ',sqrt( sum(freq**2)/nsnp - (sum(freq)/nsnp)**2 )
print *,'maxfreq= ',maxval(freq)
print *,'minfreq= ',minval(freq)
print *,'average MAF= ',sum(MAF)/nsnp
print *,'sdMAF= ',sqrt( sum(MAF**2)/nsnp - (sum(MAF)/nsnp)**2 )
print *,'maxMAF= ',maxval(MAF)
print *,'minMAF= ',minval(MAF)
write(dum,'(i2.2)') floor(t*100d0)
write(informat,'(i10)')new_nsnp
print *,informat
informat='(i10,1x,'//trim(adjustl(informat))//'i1)'
print *,informat

if(t>=0d0) then
	open(2,file=trim(adjustl(genofile))//'_MAF'//dum,status='replace')
	do i=1,nanim
	  write(2,informat)id(i),new_geno(i,:)
	enddo  
else
	print *, 'MAF=',t,'there will be no new file with genotypes'
endif
open(unit=3,file=trim(adjustl(genofile))//'_MAF'//dum//'_freq',status='replace')
do i=1,new_nsnp
  write(3,*)freq(i)
enddo  

end















